#include <mooutils/mooglade.h>
#include <mooutils/mooi18n.h>
#include <gtk/gtk.h>

static const char _log_window_xml_glade_xml[] =
"<?xml version=\"1.0\"?>"
"<glade-interface>"
"  <widget class=\"GtkWindow\" id=\"LogWindow\">"
"    <property name=\"title\">Log</property>"
"    <property name=\"default_width\">400</property>"
"    <property name=\"default_height\">300</property>"
"    <child>"
"      <widget class=\"GtkScrolledWindow\" id=\"scrolledwindow1\">"
"        <property name=\"visible\">True</property>"
"        <property name=\"can_focus\">False</property>"
"        <property name=\"hscrollbar_policy\">GTK_POLICY_AUTOMATIC</property>"
"        <property name=\"vscrollbar_policy\">GTK_POLICY_AUTOMATIC</property>"
"        <property name=\"shadow_type\">GTK_SHADOW_ETCHED_IN</property>"
"        <child>"
"          <widget class=\"GtkTextView\" id=\"textview\">"
"            <property name=\"visible\">True</property>"
"            <property name=\"wrap_mode\">GTK_WRAP_CHAR</property>"
"          </widget>"
"        </child>"
"      </widget>"
"    </child>"
"  </widget>"
"</glade-interface>"

;

typedef struct LogWindowXml LogWindowXml;

struct LogWindowXml {
    MooGladeXML *xml;

    GtkWindow *LogWindow;
    GtkTextView *textview;

};

static void
_log_window_xml_free (LogWindowXml *xml)
{
    if (xml)
    {
        if (xml->xml)
            g_object_unref (xml->xml);
        g_free (xml);
    }
}

G_GNUC_UNUSED static LogWindowXml *
log_window_xml_get (gpointer widget)
{
    return (LogWindowXml*) g_object_get_data (G_OBJECT (widget), "moo-generated-glade-xml");
}

static gboolean
_log_window_xml_finish_build (LogWindowXml *xml)
{
    g_return_val_if_fail(xml != NULL, FALSE);

    xml->LogWindow = (GtkWindow*) moo_glade_xml_get_widget (xml->xml, "LogWindow");
    g_return_val_if_fail (xml->LogWindow != NULL, FALSE);
    xml->textview = (GtkTextView*) moo_glade_xml_get_widget (xml->xml, "textview");
    g_return_val_if_fail (xml->textview != NULL, FALSE);

    g_object_set_data_full (G_OBJECT (xml->LogWindow), "moo-generated-glade-xml",
                            xml, (GDestroyNotify) _log_window_xml_free);
    return TRUE;
}

static gboolean
log_window_xml_build (LogWindowXml *xml)
{
    GError *error = NULL;
    if (!moo_glade_xml_parse_memory (xml->xml, _log_window_xml_glade_xml, -1, "LogWindow", &error))
    {
        g_critical ("Could not parse glade xml: %s", error->message);
        g_error_free (error);
        return FALSE;
    }
    return _log_window_xml_finish_build (xml);
}

static gboolean
log_window_xml_fill (LogWindowXml *xml, GtkWidget *root)
{
    GError *error = NULL;
    if (!moo_glade_xml_fill_widget (xml->xml, root, _log_window_xml_glade_xml, -1, "LogWindow", &error))
    {
        g_critical ("Could not parse glade xml: %s", error->message);
        g_error_free (error);
        return FALSE;
    }
    return _log_window_xml_finish_build (xml);
}

static LogWindowXml *
log_window_xml_new_empty (void)
{
    LogWindowXml *xml = g_new0 (LogWindowXml, 1);
    xml->xml = moo_glade_xml_new_empty (GETTEXT_PACKAGE);

    return xml;
}

G_GNUC_UNUSED static LogWindowXml *
log_window_xml_new (void)
{
    LogWindowXml *xml = log_window_xml_new_empty ();
    if (!log_window_xml_build (xml))
    {
        _log_window_xml_free (xml);
        xml = NULL;
    }
    return xml;
}

G_GNUC_UNUSED static LogWindowXml *
log_window_xml_new_with_root (GtkWidget *root)
{
    LogWindowXml *xml = log_window_xml_new_empty ();
    if (!log_window_xml_fill (xml, root))
    {
        _log_window_xml_free (xml);
        xml = NULL;
    }
    return xml;
}
